#include "GarrysMod/Lua/Interface.h"
#include "quickhull/QuickHull.hpp"
#include "mesh-subdivision/MeshSubdivision/Mesh.h"
#include "mesh-subdivision/MeshSubdivision/KobbeltRoot3.h"

/*
	lua_run require("quickhull") PrintTable(QuickHull({Vector(1,2,3),Vector(3,2,1)}))
*/

using namespace GarrysMod::Lua;

LUA_FUNCTION(QuickHull)
{
	LUA->CheckType(1, Type::Table);
	int loops;
	if (LUA->GetType(2) <= Type::Nil)
	{
		loops = 1;
	}
	else {
		loops = LUA->CheckNumber(2);
	}

	std::vector<quickhull::Vector3<float>> points;
	int n = LUA->ObjLen(1);
	for (int i = 0; i < n; i++)
	{
		LUA->PushNumber((double)i + 1);
		LUA->GetTable(1);
		Vector v = LUA->GetVector(-1);
		LUA->Pop();
		points.push_back(quickhull::Vector3<float>(v.x, v.y, v.z));
	}
	quickhull::QuickHull<float> qh;

	std::cout << "Got: " << n << " Points, Quick-Hull'ing" << std::endl;
	auto hull = qh.getConvexHull(points, true, true);
	auto verts = hull.getVertexBuffer();
	auto indices = hull.getIndexBuffer();
	std::cout << "Quickhull'd successfully, Verts: " << verts.size() << " Indices: " << indices.size() << std::endl;
	//subdivide starts here
	std::vector<quickhull::Vector3<float>> newVerts;
	std::vector<size_t> newIndices;
	for (size_t i = 0; i < indices.size(); i++)
	{
		bool exists = false;
		for (size_t j = 0; j < newVerts.size(); j++)
		{
			if (newVerts[j] == verts[indices[i]])
			{
				exists = true;
				newIndices.push_back(j);
				break;
			}
		}
		if (exists) { continue; } //if the vert already exists for this index then we've already set it, continue out to next index.
		else //if we've not found the vert, add it as a new vert and point the index. 
		{
			size_t newIndex = newVerts.size();
			newVerts.push_back(verts[indices[i]]);
			newIndices.push_back(newIndex);
		}
	}
	std::cout << "Optimizing Vectors, newVerts: " << newVerts.size() << " newIndices: " << newIndices.size() << std::endl;
	Mesh* mesh = new Mesh();
	for (size_t i = 0; i < newVerts.size(); i++)
	{
		const auto& v = newVerts[i];
		mesh->addVertex(Point(v.x, v.y, v.z));
	}
	for (size_t i = 0; i < newIndices.size(); i += 3)
	{
		size_t f[3];
		f[0] = newIndices[i + 0];
		f[1] = newIndices[i + 1];
		f[2] = newIndices[i + 2];
		mesh->addFace(f, 3);
	}
	std::cout << "Faces Calculated: " << mesh->faces.size() << ", Running Subdivision.." << std::endl;
	
	KobbeltRoot3 r3;
	for (int i = 0; i < loops; i++)
	{
		mesh = r3.run(mesh);
	}

	std::cout << "Subdivision ran successfully, Verts: " << mesh->verts.size() << " Faces: " << mesh->faces.size() << std::endl;
	LUA->CreateTable();

	for (size_t i = 0; i < mesh->faces.size(); i++)
	{
		for (size_t j = 0; j < 3; j++)
		{
			auto coord = mesh->faces[i]->verts[j]->coords;
			Vector v = Vector();
			v.x = coord.x;
			v.y = coord.y;
			v.z = coord.z;
			LUA->PushNumber((double)i*3 + j + 1);
			LUA->PushVector(v);
			LUA->SetTable(-3);
		}
		
	}
	return 1;
}
LUA_FUNCTION(TEST)
{
	LUA->PushString(__TIME__);
	return 1;
}
GMOD_MODULE_OPEN()
{
	LUA->PushSpecial(GarrysMod::Lua::SPECIAL_GLOB);
	LUA->PushString("QuickHull");
	LUA->PushCFunction(QuickHull);
	LUA->SetTable(-3);
	LUA->PushString("TEST");
	LUA->PushCFunction(TEST);
	LUA->SetTable(-3);
	return 0;
}

GMOD_MODULE_CLOSE()
{
	return 0;
}
